 /***********************************************************************
 * =======================================				*
 * EasyMaths - Core component of EasyMaths				*
 * November 2000							*
 * =======================================				*
 *									*
 * Copyright (C) 2000, 2001 Didier Davill				*
 * Developed by Didier Davill <Didier.Daville@ddarea.com>		*
 *									*
 * This library is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU Library General Public License as 	*
 * published by the Free Software Foundation; either version 2 of the	*
 * License, or (at your option) any later version.			*
 *									*
 * This library is distributed in the hope that it will be useful, but	*
 * WITHOUT ANY WARRANTY; without even the implied warranty of		*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.			*
 * See the GNU Library General Public License for more details.		*
 *									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the Free Software	*
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*
 * 02111-1307, USA.							*
 ***********************************************************************/

 /***********************************************************************
 * $Author: daville $
 *
 * $Revision: 1.3 $
 *
 * $Id: ArbreM.h,v 1.3 2000/11/02 21:33:41 daville Exp $
 * $Log: ArbreM.h,v $
 * Revision 1.3  2000/11/02 21:33:41  daville
 * Ajout du tag rvision.
 *
 * Revision 1.2  2000/11/02 21:29:02  daville
 * Ajout des enttes de coyright et de suivi de version.
 *
 ***********************************************************************/


#ifndef ARBREM_H
#define ARBREM_H
#include <stdio.h>
#include <assert.h>
#include "Liste.h"

typedef struct SNoeud
{
        void * Info;
        struct SNoeud *Pere;
        CList <struct SNoeud *> *PListFils;
}SNoeud;

template <class T> class ArbreM
{
        protected:
        SNoeud *mTete;
        SNoeud *mCourant;
        int	COP;
        void (*FONCTIONCALL)(T &Res);
        void (*AFFICHE)(T &Res);
        void (*COPIE)(T &DEST, T &SRC);
        void Create(ArbreM <T> *Arb);
        ArbreM(SNoeud *Ptr, void (*PtrD)(T&) = NULL, void (*PtrC)(T&,T&) = NULL, void (*PtrA)(T&)=NULL);
        
        public:
        void Init(void (*fcn)(T &));// Init pour bien detruire
        void InitAffiche(void (*fcn)(T &));
        void InitCopie(void (*fcn)(T &, T&));
        ArbreM();
        ArbreM(ArbreM <T> &ArbreM);
        ArbreM & operator =(ArbreM & Arb);
        int ExisteTete();
        int EstSurTete(); // Si on est au-dela de la tete.
        int EstTete();	  // Si on est bien sur la tete.
        
        int ExisteFils();
// Permet de savoir si le noeud courant  des fils.

	int GetNombreDeFils();
// Renvoie le nombre de fils que possde l'element courant
	
	int GetNombreDeFilsDuPere();
// Renvoie le nombre de fils que possede le pere	

	T* GetFils(int k);
// Renvoie le k ieme fils

	T* GetFils(T &Info);
// Permet de renvaoyer la valeur du dernier fils utilise

        T* GetFilsPremier(T &Info);
// Reste positionner sur l'lment courant et renvoie

        T* GetFilsSuivant(T &Info);
// Reste positionner sur l'lment courant et renvoie
// le fils suivant

        T* GetFilsPrecedent(T &Info);
// Reste positionner sur l'lment courant et renvoie
// le fils prcdent

        T* GetFilsDernier(T &Info);
// Reste positionner sur l'lment courant et renvoie
// le dernier fils.
	
        T* AllerFils(T &Info);
// Permet d'aller sur le dernier fils utilise

        T* AllerFilsPremier(T &Info);
// Se positionne sur le 1er fils  GAUCHE

	T* AllerFilsSuivant(T &Info);
// Se positionne sur le fils  Droite

        T* AllerFilsPrecedent(T &Info);
// Se positionne sur le fils precedent par rapport a la
// derniere postion dans la liste

        T* AllerFilsDernier(T &Info);
// Se positionne sur le dernier fils de l'element courant

        int  FinFils();
// Renvoie 0 la derniere actionn etait en postion de la 
// derniere action
        
        int  ExisteFrere();
// Renvoie 0 si l'element courant n'a plus de frere

        T* AllerFrerePremier(T &Info);
// Permet d'aller sur le premier frere.

        T* AllerFrereSuivant(T &Info);
        T* AllerFrerePrecedent(T &Info);
        T* AllerFrereDernier(T &Info);
        T* AllerPere(T &Info);
        void AllerTete(T &Info);
        int  FinFrere();
        int  WellFinFrere();
        
        void AjouterFilsDroite(T Info);
        void AjouterDroite(T Info);
        void AjouterFilsGauche(T Info);
        void AjouterGauche(T Info);

        void AjouterFils(T Info,int i=0);
// Ajoute simplement un fils et reste sur le pere
// Si i=0 Le pointeur courant de la liste restera
// sur le premier element, sinon sur le nouveau fils

        void Ajouter(T Info);
// Ajoute un fils et devient l'element courant

        void Inserer(T Info);
        void Effacer();
// Efface le sous-arbre commencant par le courant
// Le courant sera effacer aussi. Le pere devient
// ainsi l'element courant.
        
        void Supprimer();
// Supprime l'element courant et son acien pere devient
// l'element courant

	void SupprimerTete();
	void SupprimerFils();  // Supprimer tous les fils
	void Parcourir();
	ArbreM<T> *Copier();
        ArbreM<T> *Couper();
        void   Coller(ArbreM<T> *Arb);
        T*   ElementCourant(T &Info);
        T*   GetPElementCourant();
        void ElementFilsCourant(T &Info);
        void Changer(T &Src, T Dst);
        T* PositionnerFilsPremier();
        T* PositionnerFilsSuivant();
        void Detruire();
        DestrucBis();
        virtual ~ArbreM();
};

template <class T> void ArbreM <T>::Init(void (*fcn)(T &))
{
        FONCTIONCALL = fcn;
}

template <class T> void ArbreM <T>::InitAffiche(void (*fcn)(T &))
{
        AFFICHE = fcn;
}

template <class T> void ArbreM <T>::InitCopie(void (*fcn)(T &, T&))
{
        COPIE = fcn;
        COP=5;
}

template <class T> ArbreM <T>::ArbreM(SNoeud *Ptr, void (*PtrD)(T&)=NULL, void (*PtrC)(T&,T&)=NULL, void (*PtrA)(T&)=NULL)
{
        ArbreM ArbTmp,Arb;
        ArbTmp.mTete = ArbTmp.mCourant = Ptr;
        ArbTmp.FONCTIONCALL = PtrD;
        ArbTmp.COPIE = PtrC;
        ArbTmp.AFFICHE = PtrA;
        COP=ArbTmp.COP=5;
        FONCTIONCALL=ArbTmp.FONCTIONCALL;
        COPIE=ArbTmp.COPIE;
        AFFICHE=ArbTmp.AFFICHE;
        Arb=ArbTmp;
        mCourant=mTete=Arb.mTete;
        Arb.mCourant=ArbTmp.mCourant=Arb.mTete=ArbTmp.mTete=NULL;
}

template <class T> ArbreM <T>::ArbreM()
{
        mTete=NULL;
        mCourant=NULL;
        FONCTIONCALL = NULL;
        AFFICHE = NULL;
        COPIE = NULL;
}

template <class T> ArbreM<T>::ArbreM(ArbreM <T> &ArbreM)
{
        mTete=NULL;
        mCourant=NULL;
        *this=ArbreM;
}

template <class T> ArbreM<T> & ArbreM<T>::operator = (ArbreM <T> &Arb)
{
        COPIE=Arb.COPIE;
        FONCTIONCALL=Arb.FONCTIONCALL;
        AFFICHE=Arb.AFFICHE;
        if (Arb.ExisteTete())
        {
                T Info,NewInfo;
                Arb.AllerTete(Info);
                if (COPIE) (*COPIE)(NewInfo,Info); else NewInfo=Info;
                Ajouter(NewInfo);
                if (Arb.ExisteFils())
                {
        	    	Arb.AllerFilsPremier(Info);
        	    	while (!Arb.FinFrere())
                	{
                	        if (COPIE) (*COPIE)(NewInfo,Info); else NewInfo=Info;
                        	Ajouter(NewInfo);
                        	Create(&Arb);
                        	AllerPere(Info);
                        	Arb.AllerFrereSuivant(Info);
                       	}
                }
                Arb.AllerTete(Info);
                AllerTete(Info);
        } else mTete=mCourant=NULL;
        return *this;
}

template <class T> void ArbreM<T>::Create(ArbreM <T> *Arb)
{
        T Info,NewInfo;
        if (Arb->ExisteFils())
        {
                Arb->AllerFilsPremier(Info);
                while(!Arb->FinFrere())
                {
                        if (COPIE) (*COPIE)(NewInfo,Info); else NewInfo=Info;
                        Ajouter(NewInfo);
                        Create(Arb);
                        AllerPere(Info);
                        Arb->AllerFrereSuivant(Info);
                }
                Arb->AllerPere(Info);
        }
}

template <class T> int ArbreM<T>::ExisteTete()
{
        return (int)(mTete);
}

template <class T> int ArbreM<T>::EstSurTete()
{
        return (mCourant==NULL);
}

template <class T> int ArbreM<T>::EstTete()
{
        return (mCourant==mTete);
}

template <class T> int  ArbreM<T>::ExisteFils()
{
        return (mCourant->PListFils!=NULL);
}

template <class T> int ArbreM<T>::GetNombreDeFils()
{
        int n = 0;
        if (mCourant->PListFils)
        	n = (mCourant->PListFils)->GetCount();
        return n;
}

template <class T> int ArbreM<T>::GetNombreDeFilsDuPere()
{
        if (!mCourant->Pere) return 0;
        if (mCourant->Pere->PListFils) return (mCourant->Pere->PListFils)->GetCount();
        return 0;
}

template <class T> T* ArbreM<T>::GetFils(int k)
{
	SNoeud *Ptr;
	if (!mCourant->PListFils) return NULL;
	Ptr = mCourant->PListFils->Get(k);
	if (!Ptr) return NULL;
        return (T*)(Ptr->Info);
}

template <class T> T* ArbreM<T>::GetFils(T &Info)
{
	SNoeud *Ptr;
	if (!mCourant->PListFils) return NULL;
        mCourant->PListFils->Refresh(Ptr);
        Info= *((T*)(Ptr->Info));
        return (T*)(Ptr->Info);
}

template <class T> T* ArbreM<T>::GetFilsPremier(T &Info)
{
	SNoeud *Ptr;
	if (!mCourant->PListFils) return NULL;
        mCourant->PListFils->MoveFirst(Ptr);
        Info= *((T*)(Ptr->Info));
        return (T*)(Ptr->Info);
}

template <class T> T* ArbreM<T>::GetFilsSuivant(T &Info)
{
	SNoeud *Ptr;
	if (!mCourant->PListFils) return NULL;
        mCourant->PListFils->MoveNext(Ptr);
        Info= *((T*)(Ptr->Info));
        return (T*)(Ptr->Info);
}

template <class T> T* ArbreM<T>::GetFilsPrecedent(T &Info)
{
	SNoeud *Ptr;
	if (!mCourant->PListFils) return NULL;
        mCourant->PListFils->MovePrev(Ptr);
        Info= *((T*)(Ptr->Info));
        return (T*)(Ptr->Info);
}

template <class T> T* ArbreM<T>::GetFilsDernier(T &Info)
{
	SNoeud *Ptr;
	if (!mCourant->PListFils) return NULL;
        mCourant->PListFils->MoveLast(Ptr);
        Info= *((T*)(Ptr->Info));
        return (T*)(Ptr->Info);
}

template <class T> T* ArbreM<T>::AllerFils(T &Info)
{
        assert(mCourant!=NULL);
        if (!mCourant->PListFils) return;
        SNoeud *Ptr=mCourant;
        Ptr->PListFils->Refresh(mCourant);
        Info= *((T*)(mCourant->Info));
        return (T*)(mCourant->Info);
}

template <class T> T* ArbreM<T>::AllerFilsPremier(T &Info)
{
        SNoeud *Ptr;
        mCourant->PListFils->MoveFirst(Ptr);
        mCourant= Ptr;
        Info= *((T*)(mCourant->Info));
        return (T*)(mCourant->Info);
}

template <class T> T* ArbreM<T>::AllerFilsSuivant(T &Info)
{
        SNoeud *Ptr;
        mCourant->PListFils->MoveNext(Ptr);
        mCourant= Ptr;
        Info= *((T*)(mCourant->Info));
	return (T*)(mCourant->Info);
}

template <class T> T* ArbreM<T>::AllerFilsPrecedent(T &Info)
{
        SNoeud *Ptr;
        mCourant->PListFils->MovePrev(Ptr);
        mCourant= Ptr;
        Info= *((T *)(mCourant->Info));
        return (T*)(mCourant->Info);
}

template <class T> T* ArbreM<T>::AllerFilsDernier(T &Info)
{
        SNoeud *Ptr; 
        mCourant->PListFils->MoveLast(Ptr);
        mCourant= Ptr;
        Info= *((T*)(mCourant->Info));
        return (T*)(mCourant->Info);
}

template <class T> int  ArbreM<T>::FinFils()
{
        return mCourant->PListFils->Eol();
}

template <class T> int  ArbreM<T>::ExisteFrere()
{
        T Info;
        int i;
        SNoeud *Ptr=mCourant->Pere,*PtrRes;
        Ptr->PListFils->MoveNext(PtrRes);
        i=Ptr->PListFils->Eol();
        Ptr->PListFils->MovePrev(PtrRes);
        return i;
}

template <class T> T* ArbreM<T>::AllerFrerePremier(T &Info)
{
        SNoeud *Ptr;
        mCourant=mCourant->Pere;
        mCourant->PListFils->MoveFirst(Ptr);
        mCourant=Ptr;
        Info= *((T*)(mCourant->Info));
        return (T*)(mCourant->Info);
}

template <class T> T* ArbreM<T>::AllerFrereSuivant(T &Info)
{
        SNoeud *Ptr,*P;
        Ptr=mCourant->Pere;
        assert(Ptr!=NULL);
        assert(Ptr->PListFils->Eol()==0);
        Ptr->PListFils->MoveNext(P);
        if (Ptr->PListFils->Eol()) return NULL;
        mCourant=P;
        Info= *((T*)(mCourant->Info));
        return (T*)(mCourant->Info);
}

template <class T> T* ArbreM<T>::AllerFrerePrecedent(T &Info)
{
        SNoeud *Ptr;
        Ptr=mCourant->Pere;
        assert(Ptr!=NULL);
        Ptr->PListFils->MovePrev(mCourant);
        Info= *((T*)(mCourant->Info));
        return (T*)(mCourant->Info);
}

template <class T> T* ArbreM<T>::AllerFrereDernier(T &Info)
{
        SNoeud *Ptr;
        mCourant=mCourant->Pere;
        mCourant->PListFils->MoveLast(Ptr);
        mCourant=Ptr;
        Info= *((T*)(mCourant->Info));
        return (T*)(mCourant->Info);
}

template <class T> T* ArbreM<T>::AllerPere(T &Info)
{
        if (mCourant)
        {
                if (mCourant->Pere)
                {
                        mCourant= mCourant->Pere;
                        Info= *((T*)(mCourant->Info));
                        return (T*)(mCourant->Info);
                }return NULL;
      	}return NULL;
}

template <class T> void ArbreM<T>::AllerTete(T &Info)
{
        if (mTete)
        {
                mCourant= mTete;
        	Info= *((T*)(mCourant->Info));
      	}
}

template <class T> int  ArbreM<T>::WellFinFrere()
{
        int i;
        SNoeud *Ptr,*Ptr2;
        Ptr=mCourant->Pere;
        if (Ptr) i=Ptr->PListFils->WellEol(); else i=!0;
        return i;
}

template <class T> int  ArbreM<T>::FinFrere()
{
        int i;
        SNoeud *Ptr,*Ptr2;
        Ptr=mCourant->Pere;
        if (Ptr) i=Ptr->PListFils->Eol(); else i=!0;
        return i;
}

template <class T> void ArbreM<T>::AjouterFilsDroite(T Info)
{
        SNoeud *PSNoeu= new SNoeud;/* La nouvelle cellule */
        T *PtrInfo=new T;
        assert(PSNoeu!=NULL);
        assert(PtrInfo!=NULL);
        PSNoeu->Pere=mCourant;
        *PtrInfo=Info;
        PSNoeu->Info= (void *)PtrInfo;
        PSNoeu->PListFils=NULL;
        if (mTete==NULL) mTete=mCourant; else
        {
                if (!mCourant->PListFils) mCourant->PListFils=new CList<SNoeud *>;
                mCourant->PListFils->Ins(PSNoeu);
        }
}

template <class T> void ArbreM<T>::AjouterDroite(T Info)
{
        AjouterFilsDroite(Info);
        AllerFilsPrecedent(Info);
}

template <class T> void ArbreM<T>::AjouterFilsGauche(T Info)
{
        SNoeud *PSNoeu= new SNoeud;/* La nouvelle cellule */
        T *PtrInfo=new T;
        assert(PSNoeu!=NULL);
        assert(PtrInfo!=NULL);
        SNoeud *Ptr;
        PSNoeu->Pere=mCourant;
        *PtrInfo=Info;
        PSNoeu->Info= (void *)PtrInfo;
        PSNoeu->PListFils= NULL;
        if (mTete==NULL) mTete=PSNoeu; else
        {
                if (!mCourant->PListFils) mCourant->PListFils=new CList<SNoeud *>;
                mCourant->PListFils->MoveNext(Ptr);
                mCourant->PListFils->Ins(PSNoeu);
        }
        if (mCourant->PListFils->Eol()) mCourant->PListFils->MoveLast(Ptr);
        mCourant->PListFils->MovePrev(PSNoeu);
}

template <class T> void ArbreM<T>::AjouterGauche(T Info)
{
        AjouterFilsGauche(Info);
        if (mCourant->PListFils->GetCount()==1) AllerFilsPremier(Info);
        	else AllerFilsSuivant(Info);
}

template <class T> void ArbreM<T>::AjouterFils(T Info,int i=0)
{
        SNoeud *PSNoeu= new SNoeud;/* La nouvelle cellule */
        T *PtrInfo=new T;
        assert(PSNoeu!=NULL);
        assert(PtrInfo!=NULL);
        PSNoeu->Pere=mCourant;
        *PtrInfo=Info;
        PSNoeu->Info= (void *)PtrInfo;
        PSNoeu->PListFils= NULL;
        if (mTete==NULL)
        {
                mTete=PSNoeu;
                mCourant=mTete;
        }
        else
        {
                if (!mCourant->PListFils) mCourant->PListFils=new CList<SNoeud *>;
                if (i) mCourant->PListFils->AddAndGo(PSNoeu);
                	else mCourant->PListFils->Add(PSNoeu);
        }       
}

template <class T> void ArbreM<T>::Ajouter(T Info)
{
        SNoeud *PSNoeu= new SNoeud;/* La nouvelle cellule */
        T *PtrInfo=new T;
        assert(PSNoeu!=NULL);
        assert(PtrInfo!=NULL);
        PSNoeu->Pere=mCourant;
        *PtrInfo=Info;
        PSNoeu->Info= (void *)PtrInfo;
        PSNoeu->PListFils= NULL;
        if (mTete==NULL) mTete=PSNoeu; else
        {
                if (!mCourant->PListFils) mCourant->PListFils=new CList<SNoeud *>;
                mCourant->PListFils->AddAndGo(PSNoeu);
        }
        mCourant=PSNoeu;
}     

template <class T> void ArbreM<T>::Inserer(T Info)
{
        if (!mCourant)
        {
                AjouterFils(Info);
                return;
        }
        SNoeud *Ptr,*PSNoeu=new SNoeud;
        T *PtrT;
        PSNoeu->Pere=mCourant->Pere;
        PtrT=new T;
        *PtrT=Info;
        PSNoeu->Info=(void *)PtrT;
        PSNoeu->PListFils=new CList<SNoeud *>;
        PSNoeu->PListFils->Add(mCourant);
        Ptr=mCourant->Pere;
        if (Ptr) Ptr->PListFils->Replace(mCourant,PSNoeu);
        mCourant->Pere= PSNoeu;
        if (mCourant==mTete) mTete=PSNoeu;
        mCourant=PSNoeu;
}

template <class T> void ArbreM<T>::Effacer()
{
        ArbreM <T> *PA;
        if (PA=Couper()) delete PA;
}

template <class T> void ArbreM<T>::Supprimer()
{
        assert(mCourant!=NULL);
        if (!mCourant->Pere) return;
        
        SNoeud **TabPtr,*Ptr;
        CList <SNoeud *>*P;
        int n,i;
        
        P=mCourant->PListFils;
        if (P)
        {
                n=P->GetCount();
                TabPtr=new (SNoeud *)[n+1];
                TabPtr[n]=NULL;
                if (n)
                {
                        P->MoveFirst(Ptr);
                	for (i=0;!P->Eol();i++)
                	{
                 	       TabPtr[i]=Ptr;
                     	       TabPtr[i]->Pere=mCourant->Pere;
                 	       P->MoveNext(Ptr);
                	}
                }
                Ptr=mCourant;
                mCourant=mCourant->Pere;
                if (!mCourant->PListFils) mCourant->PListFils=new CList<SNoeud *>;
                for(i=0;TabPtr[i];i++)
                {
                        mCourant->PListFils->Ins(TabPtr[i]);
                        mCourant->PListFils->MoveNext(Ptr);
          	}
          	delete TabPtr;
          }
          mCourant=Ptr;
          Detruire();  // Detruit l'elt cour et le courant sera le pere
}

template <class T> void ArbreM<T>::SupprimerFils()
{
        if (mCourant->PListFils)
        {
                SNoeud *Ptr;
                mCourant->PListFils->MoveFirst(Ptr);
                while (mCourant->PListFils)
        	{
                	mCourant=Ptr;
                	Detruire();
                	if (mCourant->PListFils) mCourant->PListFils->MoveFirst(Ptr);
        	}
        }
}

template <class T> void ArbreM<T>::SupprimerTete()
{
        assert(mTete!=NULL);
        int n=0;
        if (mTete->PListFils) n=mTete->PListFils->GetCount();
        if (n>=2) return;
        if (n)
        {
                T Inf;
                AllerTete(Inf);
                AllerFilsPremier(Inf);
                mCourant->Pere=NULL;
                delete mTete->PListFils;
        } else mCourant=NULL;
        if (FONCTIONCALL) (*FONCTIONCALL)(*((T *)(mTete->Info)));
        delete ((T *)(mTete->Info));
        mTete=mCourant;
}


template <class T> ArbreM<T> *ArbreM<T>::Copier()
{
        ArbreM<T> A,*PA;
        A.mTete=mCourant;
        A.mCourant=mCourant;
        A.FONCTIONCALL=FONCTIONCALL;
        A.AFFICHE=AFFICHE;
        A.COPIE=COPIE;
        PA=new ArbreM<T>(A);
        A.mTete=A.mCourant=NULL;
        return PA;
}

template <class T> ArbreM<T> *ArbreM<T>::Couper()
{
        if (!mTete) return NULL;
                
        SNoeud *P,*Ptr=mCourant;
        if (mCourant==mTete)
        {
                Ptr=mCourant;
                mCourant=NULL;
                mTete=NULL;
        }
        else
        {
                P=mCourant->Pere;
        	P->PListFils->Del(Ptr);
        	if (P->PListFils->Eol())// P->PListFils->Refresh(mCourant); else
        	{
        	       delete (P->PListFils);
        	       P->PListFils = NULL;
        	       //mCourant = P;
        	}
        	Ptr->Pere=NULL;
        	mCourant->Pere = NULL;
        	mCourant = P;
        }
        ArbreM<T> *PArbreTmp=new ArbreM<T>;
        PArbreTmp->mCourant=Ptr;
	PArbreTmp->mTete=Ptr;
	PArbreTmp->FONCTIONCALL = FONCTIONCALL;
	PArbreTmp->AFFICHE = AFFICHE;
	PArbreTmp->COPIE = COPIE;
	return PArbreTmp;
}

template <class T> void ArbreM<T>::Coller(ArbreM<T> *Arb)
{
        assert(Arb!=NULL);
        assert(Arb->mTete!=NULL);
        if (!mTete)
        {
                mTete=Arb->mTete;
                mCourant=mTete;
        } else
        {
                if (!mCourant->PListFils)
                	mCourant->PListFils=new CList<SNoeud *>;
                mCourant->PListFils->Add(Arb->mTete);
                Arb->mTete->Pere=mCourant;
        }
        Arb->mTete=NULL;
        Arb->mCourant=NULL;
}

template <class T> T* ArbreM<T>::ElementCourant(T &Info)
{
        SNoeud *Ptr;
        Info=*((T *)(mCourant->Info));
        return ((T *)mCourant->Info);
}

template <class T> void ArbreM<T>::ElementFilsCourant(T &Info)
{
        SNoeud *Ptr;
        mCourant->PListFils->Refresh(Ptr);
        Info=*((T *)(Ptr->Info));
}

template <class T> T*   ArbreM<T>::GetPElementCourant()
{
        return ((T *)mCourant->Info);
}

template <class T> void ArbreM<T>::Changer(T &Src, T Dst)
{
        assert(mCourant!=NULL);
        T *Ptr;
        delete ((T *)mCourant->Info);
        Ptr=new T;
        *Ptr=Dst;
        mCourant->Info=((void *)Ptr);
}

template <class T> T* ArbreM<T>::PositionnerFilsPremier()
{
        SNoeud *Ptr;
        if (!mCourant->PListFils) return 0;
        mCourant->PListFils->MoveFirst(Ptr);
        return (T *)(ptr->Info);
}

template <class T> T* ArbreM<T>::PositionnerFilsSuivant()
{
        SNoeud *Ptr;
        if (!mCourant->PListFils) return 0;
        mCourant->PListFils->MoveNext(Ptr);
        return (T *)(ptr->Info);
}

template <class T> void ArbreM<T>::Detruire()
{
        if (mCourant)
        {
                SNoeud *P;
                if (FONCTIONCALL) (*FONCTIONCALL)(*((T *)(mCourant->Info)));
                delete((T *)(mCourant->Info));
                if (mCourant->PListFils) delete mCourant->PListFils;
                mCourant->PListFils=NULL;
                P=mCourant->Pere;
                if (P)
                {
                        P->PListFils->Del(mCourant);
                        if (!P->PListFils->GetCount())
                        {
                                delete P->PListFils;
                                P->PListFils = NULL;
                        }
               	}
                delete mCourant;
                mCourant=P;
        }
}

template <class T> ArbreM<T>::~ArbreM()
{
        if (mTete)
        {
                if (mTete->PListFils)
                {
                        int n;
                        mTete->PListFils->MoveFirst(mCourant);
                        n=mTete->PListFils->GetCount();
                        if (AFFICHE) (*AFFICHE)(*((T *)(mCourant->Info)));
                        while (mTete->PListFils)
                        {
                                DestrucBis();
                                if (mTete->PListFils) mTete->PListFils->MoveFirst(mCourant);
                        }
                }
                mCourant=mTete;
                Detruire();
                mTete=NULL;
      	}
}

template <class T> void ArbreM<T>::Parcourir()
{
        if (!mTete)
        {
                printf("\nERREUR: Arbre vide!\n");
                return;
        }
        if (!AFFICHE)
        {
                printf("\nERREUR: Fonction d'affichage non initialisee\n");
                return;
        }
        char c;
        int i;
        T inf;
        printf("<<<<<<---  VALEUR DE LA TETE: %p   --->>>>>>>\n",mTete);
        printf("Parcours de l'arbre. Les commades:\n");
printf("	0	Racine(Tete de l'arbre)
        	2	Premier fils
        	4	Frere gauche
        	6	Frere droite
        	8	Pere
        	q	Quitter\n");
        getchar();
        while (1)
        {
           c=getchar();
           if (mCourant) i=(mCourant->PListFils!=NULL); else i=0;
           if (c=='0') AllerTete(inf); else
           {
             if (c=='4') {AllerFrerePrecedent(inf);} else
             {
               if (c=='6') {if (!FinFrere()) AllerFrereSuivant(inf);} else
               {
                 if (c=='2') {if (i) AllerFilsPremier(inf);} else
                 {
                    if (c=='8') {if (mCourant->Pere) AllerPere(inf);} else
                    {
                        if (c=='q') return;
                    }
                 }
               }
             }
           }
           (*AFFICHE)(*((T *)(mCourant->Info)));
        }
}

template <class T> ArbreM<T>::DestrucBis()
{
        SNoeud *Ptr,*Ptr2;
        Ptr2=mCourant;
        if (mCourant->PListFils)
        {
                mCourant->PListFils->MoveFirst(Ptr);
                while (mCourant->PListFils)
        	{
                	mCourant=Ptr;
                	DestrucBis();
                	if (mCourant->PListFils) mCourant->PListFils->MoveFirst(Ptr);
        	}
        }
        Detruire();
}

#endif