 /***********************************************************************
 * =======================================				*
 * EasyMaths - Core component of EasyMaths				*
 * November 2000							*
 * =======================================				*
 *									*
 * Copyright (C) 2000, 2001 Didier Davill				*
 * Developed by Didier Davill <Didier.Daville@ddarea.com>		*
 *									*
 * This library is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU Library General Public License as 	*
 * published by the Free Software Foundation; either version 2 of the	*
 * License, or (at your option) any later version.			*
 *									*
 * This library is distributed in the hope that it will be useful, but	*
 * WITHOUT ANY WARRANTY; without even the implied warranty of		*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.			*
 * See the GNU Library General Public License for more details.		*
 *									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the Free Software	*
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*
 * 02111-1307, USA.							*
 ***********************************************************************/

 /***********************************************************************
 * $Author: daville $
 *
 * $Revision: 1.2 $
 *
 * $Id: intervalle.cpp,v 1.2 2000/11/02 22:10:22 daville Exp $
 * $Log: intervalle.cpp,v $
 * Revision 1.2  2000/11/02 22:10:22  daville
 * Intgration de l'entte de copyright et de suivi de version.
 *
 *
 ***********************************************************************/


#include "intervalle.h"

XMIntervalle::XMIntervalle()
{
	mType = TO_INTERVALLE;
	mBorne[0] = mBorne[1] =NULL;
	mX = new XMVariable();
	mStep = 0.08;
}

XMIntervalle::XMIntervalle(XMVariable *inf, XMVariable *sup)
{
	mType = TO_INTERVALLE;
	mBorne[0] = inf;
	mBorne[1]= sup;
	mX = (XMVariable *)inf->Clone();
	mStep = 0.08;
}

XMIntervalle::XMIntervalle(char *name, XMVariable *inf, XMVariable *sup)
{
	mType = TO_INTERVALLE;
	mBorne[0] = inf;
	mBorne[1]= sup;
	mX = (XMVariable *)inf->Clone();
	SetName(name);
	mStep = 0.08;
}

XMIntervalle::XMIntervalle(double min, double max)
{
	char mins[50], maxs[50];
	mType = TO_INTERVALLE;
	sprintf(mins, "%fd", min);
	sprintf(maxs, "%fd", max);
	mBorne[0] = new XMVariable(NULL, mins, min);
	mBorne[1] = new XMVariable(NULL, maxs, max);
	mX = (XMVariable *)mBorne[0]->Clone();
	mStep = 0.08;
}

char *XMIntervalle::Formatted()
{
	char *formatted;
	int v;
	formatted = new char[200];
	sprintf(formatted, "%d#%s#%s..%s#%f..%f#", mType, mName,
			mBorne[0]->EvalF(v, NULL), mBorne[1]->EvalF(v, NULL),
			mBorne[0]->Eval(v, NULL),  mBorne[1]->Eval(v, NULL));
	return formatted;
}

TYPE XMIntervalle::Inc()
{
	int v;
	mX->SetValue(mStep+mX->Eval(v));
	return mX->Eval(v);
}

TYPE XMIntervalle::Dec()
{
	int v;
	mX->SetValue(mX->Eval(v)-mStep);
	return mX->Eval(v);
}	

void XMIntervalle::Init()
{
	int v;
	mX->SetValue(mBorne[0]->Eval(v));
	XMTRACE("Appelle a Init %f\n",mX->Eval(v));
};

bool XMIntervalle::IsValid()
{
	int v;
	return ((mX->Eval(v)>=mBorne[0]->Eval(v)) && (mX->Eval(v)<=mBorne[1]->Eval(v)));
}

bool XMIntervalle::Containt(double pt)
{
	int v;
	return ((pt>=mBorne[0]->Eval(v)) && (pt<=mBorne[1]->Eval(v)));
}

TYPE  XMIntervalle::Eval(int &i, CList <XMObjectX *> *lvar=NULL)
{
	return mX->Eval(i);
}

XMIntervalle::~XMIntervalle()
{
	if (mBorne[0]) delete mBorne[0];
	if (mBorne[1]) delete mBorne[1];
	delete mX;
	mX = mBorne[0] = mBorne[1] = NULL;
	mType = TO_UNKNOWN;
}