 /***********************************************************************
 * =======================================				*
 * EasyMaths - Core component of EasyMaths				*
 * November 2000							*
 * =======================================				*
 *									*
 * Copyright (C) 2000, 2001 Didier Davill				*
 * Developed by Didier Davill <Didier.Daville@ddarea.com>		*
 *									*
 * This library is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU Library General Public License as 	*
 * published by the Free Software Foundation; either version 2 of the	*
 * License, or (at your option) any later version.			*
 *									*
 * This library is distributed in the hope that it will be useful, but	*
 * WITHOUT ANY WARRANTY; without even the implied warranty of		*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.			*
 * See the GNU Library General Public License for more details.		*
 *									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the Free Software	*
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*
 * 02111-1307, USA.							*
 ***********************************************************************/

 /***********************************************************************
 * $Author: daville $
 *
 * $Revision: 1.2 $
 *
 * $Id: matrice.cpp,v 1.2 2000/11/02 22:10:08 daville Exp $
 * $Log: matrice.cpp,v $
 * Revision 1.2  2000/11/02 22:10:08  daville
 * Intgration de l'entte de copyright et de suivi de version.
 *
 *
 ***********************************************************************/


#include <math.h>
#include <stdlib.h>
#include "matrice.h"
#include "number.h"

XMMatrice::XMMatrice()
{
        mType = TO_MATRICE;
        mMat=NULL;
        NbreLig=NbreCol=0;
}

XMMatrice::XMMatrice(XMMatrice &M)
{
        int i,j;
        mType = TO_MATRICE;
        mMat = new (XMVecteur *)[M.NbreCol];
        for(j=0;j<NbreCol;j++) mMat[j] = new XMVecteur(M.NbreLig);
        NbreLig=M.NbreLig; NbreCol=M.NbreCol;
        for(j=0;j<NbreCol;j++) for(i=0; i<NbreLig; i++) mMat[j]->SetCoord(M.mMat[j]->GetCoord(i),i);
}

XMMatrice::XMMatrice(int NL, int NC)
{
        int i,j;
        mType = TO_MATRICE;
        mMat=new (XMVecteur *)[NC];
        for(j=0;j<NC;j++) mMat[j]=new XMVecteur(NL);
        NbreLig=NL; NbreCol=NC;
        for(j=0;j<NC;j++) for(i=0;i<NL;i++) mMat[j]->SetCoord(NULL,i);
}

void XMMatrice::Set(int i, int j, XMObjectX *obj)
{
        if ((i>NbreLig) || (j>NbreCol)) return;
        mMat[j]->SetCoord(obj,i);
}

void XMMatrice::Set(int j, XMVecteur *vect)
{
	if (j>NbreCol) return;
	if (mMat[j]) delete (mMat[j]);
	mMat[j] = vect;
}

XMMatrice *XMMatrice::Mineure(int io, int jo)
{
        XMMatrice *M=new XMMatrice(NbreLig-1,NbreCol-1);
        int i,j,k,l;
        for(j=0;j<M->NbreCol;j++) for (i=0;i<M->NbreLig;i++)
        {
                if (i<io) k=i; else k=i+1;
		if (j<jo) l=j; else l=j+1;
		(M->mMat[j])->SetCoord(mMat[k]->GetCoord(l)->Clone(),i);
	}
        return M;
}

float XMMatrice::Det()
{
        int i,k;
        if (NbreLig*NbreCol==1) return (mMat[0]->GetCoord(0))->Eval(i);
        float S=0;
        XMMatrice *M;
        for (k=0;k<NbreLig;k++)
        {
                M = Mineure(k,0);
                S = S+pow(-1,k)*(mMat[k]->GetCoord(0))->Eval(i)*M->Det();
                delete M;
        }
        return S;
}

char *XMMatrice::DetF()
{
        int i;
        if (NbreLig*NbreCol==1) return mMat[0]->GetCoord(0)->EvalF(i);
        char *S=NULL;
        char *Res[10],ks[20],*m="-1";
        TYPE v;
        XMMatrice *M;
        for (int k=0;k<NbreLig;k++)
        {
                M = Mineure(k,0);
                Res[0] = M->DetF();
                gcvt(k,15,ks);
                Res[1] = Simplifie_Power(m,ks);
                Res[2] = mMat[k]->GetCoord(0)->EvalF(i);
                Res[3]=Simplifie_Multi(Res[1],Res[2]);
                Res[4]=Simplifie_Multi(Res[0],Res[3]);
                if (!S)
                {
                        S=new char[strlen(Res[4])+1];
                        strcpy(S,Res[4]);
                } else
                {
                        Res[0]=S;  S=Simplifie_Somme(Res[0],Res[4]);
                }
                free(Res[0]);free(Res[1]);
                free(Res[3]);free(Res[4]);free(Res[2]);
                delete M;
        }
        return S;
}

XMMatrice *XMMatrice::Inv()
{
        TYPE DET = Det(),Res;
        if (!DET) return NULL;
        int i,j;
        XMMatrice *M = Compl();
        //for(i=0;i<NbreLig;i++) for(j=0;j<NbreCol;j++)
        return M;
}

XMMatrice *XMMatrice::Compl()
{
        XMMatrice *M = Com();
        XMMatrice *Mo = M->Trans();
        delete M;
        return Mo;
}

XMMatrice *XMMatrice::Com()
{
        XMMatrice *M=new XMMatrice(NbreLig,NbreCol);
        XMMatrice *P;
        XMObjectX *F;
        int i,j, k;
        char js[1];
        char *Res[3],ks[10],is[2],*m="-1"; // si m^ lign is et js => bizzare
        for (i=0;i<+NbreLig;i++)
        {
                 for (j=0;j<NbreCol;j++)
                 {
                        P = Mineure(i,j);
                        Res[0]=P->DetF();
                        k=i+j;
                	gcvt(k,2,ks);
                	Res[1]=Simplifie_Power(m,ks);
                	Res[2]=Simplifie_Multi(Res[1],Res[0]);
                	gcvt(i,1,is);
                	gcvt(j,1,js);
                	strcat(is,js);
                	
                	
                	//F=new Fonction(is,Res[2]);
                	
                	
                	M->Set(i,j,F);
                	delete Res[0]; delete Res[1]; delete Res[2];
              	 }
        }
        return M;
}

XMMatrice *XMMatrice::Trans()
{
        XMMatrice *M=new XMMatrice(NbreCol,NbreLig);
        for (int i=0;i<NbreLig;i++) for(int j=0;j<NbreCol;j++) M->mMat[j]->SetCoord(mMat[i]->GetCoord(j)->Clone(),i);
        return M;
}

char *XMMatrice::Formatted()
{
	char *formatted;
	
	formatted = new char[1000];
	sprintf(formatted, "%d#%s#", mType, mName);
	return formatted;
}

void XMMatrice::Lecture()
{
        int vld;
        printf("Nombre de colonne: %d\n",NbreCol);
        printf("Nombre de ligne: %d\n",NbreLig);
        printf("Nombre de ligne d'un vecteur: %d\n",mMat[0]->GetDim());
        for (int i=0; i<NbreLig; i++)
        {
        	for (int j=0; j<NbreCol; j++)
        		if (mMat[j]->GetCoord(i))
        			printf(" %f	", (mMat[j]->GetCoord(i))->Eval(vld));
        	printf("\n");
        }
}

XMMatrice::~XMMatrice()
{
        int i,j;char c;
        for(j=0;j<NbreCol;j++)
    	{
    	   if (mMat[j]) delete mMat[j];
           mMat[j]=NULL;
        }
        delete mMat;
}