 /***********************************************************************
 * =======================================				*
 * EasyMaths - Core component of EasyMaths				*
 * November 2000							*
 * =======================================				*
 *									*
 * Copyright (C) 2000, 2001 Didier Davill				*
 * Developed by Didier Davill <Didier.Daville@ddarea.com>		*
 *									*
 * This library is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU Library General Public License as 	*
 * published by the Free Software Foundation; either version 2 of the	*
 * License, or (at your option) any later version.			*
 *									*
 * This library is distributed in the hope that it will be useful, but	*
 * WITHOUT ANY WARRANTY; without even the implied warranty of		*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.			*
 * See the GNU Library General Public License for more details.		*
 *									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the Free Software	*
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*
 * 02111-1307, USA.							*
 ***********************************************************************/

 /***********************************************************************
 * $Author: daville $
 *
 * $Revision: 1.2 $
 *
 * $Id: ploter.cpp,v 1.2 2000/11/02 22:09:52 daville Exp $
 * $Log: ploter.cpp,v $
 * Revision 1.2  2000/11/02 22:09:52  daville
 * Intgration de l'entte de copyright et de suivi de version.
 *
 *
 ***********************************************************************/


#include "ploter.h"

XMPloter::XMPloter()
{
	mType = TO_PLOTER;
	mMethod = NULL;
	mAPloter = NULL ;
	mIntervalles = NULL;
}

XMPloter::XMPloter(char *smeth)
{
	mType = TO_PLOTER;
	TMethod meth=GetTypeMethodFor(smeth);
	if (meth == TM_UNKNOWN) meth = TM_CARTESIAN2D;
	mMethod = GetMethodFor(meth);
	mAPloter = NULL ;
	mIntervalles = NULL;
}

XMPloter::XMPloter(TMethod meth)
{
	mType = TO_PLOTER;
	mMethod = GetMethodFor(meth);
	if (!mMethod) mMethod = GetMethodFor(TM_CARTESIAN2D);
	mAPloter = NULL ;
	mIntervalles = NULL;
}

XMPloter::XMPloter(XMObjectX *ob, TMethod meth=TM_CARTESIAN2D)
{
	mType = TO_PLOTER;
	mMethod = GetMethodFor(meth);
	mIntervalles = NULL;
	mAPloter = new CList<XMObjectX *>;
	mAPloter->Add(ob);
}

XMPloter::XMPloter(TMethod meth, CList<XMObjectX *> *pobj, CList<XMIntervalle *> *inter)
{
	mType = TO_PLOTER;
	mMethod = GetMethodFor(meth);
	mAPloter = pobj;		// Doit terminer par null
	mIntervalles = inter;		// Doit terminer par null
}

void XMPloter::Add(XMObjectX *obj)
{
	if (obj->Type() == TO_INTERVALLE)
	{
		if (!mIntervalles) mIntervalles = new CList<XMIntervalle *>;
		mIntervalles->Add((XMIntervalle *)obj);
	} else
	{
		if (!mAPloter) mAPloter = new CList<XMObjectX *>;
		mAPloter->Add(obj);
	}
}

void *XMPloter::Map(CList <XMObjectX *> *lvar)
{
	XMTRACE("Appelle  Ploter::Map\n");
	return (mMethod?mMethod->Map(mAPloter,lvar):NULL);
}

void XMPloter::SetMethode(TMethod meth)
{
	int n;
	mMethod = GetMethodFor(meth);
}

int XMPloter::Dim()
{
	return mMethod->Dim();
}

CList <XMIntervalle *> *XMPloter::Intervalles()
{
	return mIntervalles;
}

XMMethodPlot * XMPloter::Method()
{
	return mMethod;
}

TYPE ** XMPloter::Reach(int &vld, TYPE **pt, TYPE **Res)
{
	//mMethod->Reach(vld, mAPloter, pt, Res);
	return Res;
}

void  XMPloter::Lecture()
{
	XMTRACE("==========  Lecture de l'objet XMPloter =============\n");
	XMTRACE("Type: %d\n", mType);
	if (mMethod)
	{
		XMTRACE("Methode\n");
		XMTRACE(" Type: %d\n", mMethod->Method());
 		XMTRACE(" Dimension: %d\n", mMethod->Dim());
	} else XMTRACE("Methode: Acune!!!\n");
	XMTRACE("Ploter\n");
	if (mAPloter) XMTRACE("Nbre d'objets  tracer: %d\n", mAPloter->GetCount()); else
		XMTRACE("Aucun objet  tracer\n");
	if (mIntervalles) XMTRACE("Nbre d'objets  tracer: %d\n",mIntervalles->GetCount()); else
		XMTRACE("Aucun objet  tracer\n");
	mIntervalles = NULL;
}

XMPloter::~XMPloter()
{
	if (mMethod) delete mMethod;
	if (mAPloter) delete mAPloter;
	if (mIntervalles) delete mIntervalles;
	mMethod = NULL;
	mAPloter = NULL ;
	mIntervalles = NULL;
}