 /***********************************************************************
 * =======================================				*
 * EasyMaths - Core component of EasyMaths				*
 * November 2000							*
 * =======================================				*
 *									*
 * Copyright (C) 2000, 2001 Didier Davill				*
 * Developed by Didier Davill <Didier.Daville@ddarea.com>		*
 *									*
 * This library is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU Library General Public License as 	*
 * published by the Free Software Foundation; either version 2 of the	*
 * License, or (at your option) any later version.			*
 *									*
 * This library is distributed in the hope that it will be useful, but	*
 * WITHOUT ANY WARRANTY; without even the implied warranty of		*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.			*
 * See the GNU Library General Public License for more details.		*
 *									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the Free Software	*
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*
 * 02111-1307, USA.							*
 ***********************************************************************/

 /***********************************************************************
 * $Author: daville $
 *
 * $Revision: 1.2 $
 *
 * $Id: type.h,v 1.2 2000/11/02 22:09:51 daville Exp $
 * $Log: type.h,v $
 * Revision 1.2  2000/11/02 22:09:51  daville
 * Intgration de l'entte de copyright et de suivi de version.
 *
 *
 ***********************************************************************/


#ifndef MTYPE_H
#define MTYPE_H
#define TYPE double
#define XMTRACE printf

/********************	TReponse    **************/
//Les diffrentes possibilits pour une rponse.

enum TReponse{
        TR_ERROR,
        TR_EXIT,
        TR_INFO,
        TR_OK,
        TR_VOID};
        
/********************  TObjectX	  ****************/
// Les diffrents types d'objet mathmatiques

enum TObjectX{
        TO_FONCTION,		//0
        TO_VECTEUR,		//1
        TO_MATRICE,		//2
        TO_VARIABLE,		//3
        TO_POLYNOME,		//4
        TO_INTERVALLE,		//5
        TO_PLOT,		//6
        TO_PLOTER,		//7
        TO_SYSCOORD,		//8
        TO_PLOT2D,		//9
        TO_PLOT3D,		//10
        TO_SCRIPT,		//11
        TO_KEYWORD,		//12
        TO_WHOLE,		//13
        TO_HEAD,		//14
        TO_INIT,		//15
        TO_UNKNOWN		//16
        };

/********************  TOperator  ****************/
// Les diffrents types d'oprateur
           
enum TOperator{
        TOP_ABS = TO_UNKNOWN + 1,           // 17, 18, ....
        TOP_CROCHETF,
        TOP_CROCHETO,
        TOP_DIVISE,
        TOP_EGAL,
        TOP_EXP,
        TOP_HYPER,
        TOP_LN,
        TOP_MOINS,
        TOP_MULTIPLIE,
        TOP_PARENTHESEF,
        TOP_PARENTHESEO,
        TOP_PLUS,
        TOP_PWD,
        TOP_TRIGO,
        TOP_INTERVALLE,
        TOP_VARIABLE,
        TOP_VIRGULE,
        TOP_WHOLE,
        TOP_CAST,

        TOP_LAST,	/* Fin des operateurs standards */
        
        TNSOP_DET,
        TNSOP_DIFF,
        TNSOP_EVAL,
        TNSOP_EVALF,
        TNSOP_MAT,
	TNSOP_PLOT,
        TNSOP_TEST,
        
        TNSOP_COMM,
        TOP_UNKNOWN,
        TOP_STRICT_INF,
        TOP_STRICT_SUP,
        TOP_INF_OU_EGAL,
        TOP_SUP_OU_EGAL,
        TOP_EGAL_EGAL,
        TNSOP_LAST
        };
        

enum TMethod{
	TM_CARTESIAN2D,
	TM_PARAMETRIC2D,
	TM_POLAR,
	TM_CARTESIAN3D,
	TM_PARAMETRIC3D,
	TM_CYLINDRIC,
	TM_SPHERIC,
	TM_UNKNOWN
	};
	
static int XM_DEFAULT_SIGNIFICATIF=-1;
	
enum TValueX{
	TV_SHORT,		// 0
	TV_INT,			// 1
	TV_LONG_INT,		// 2
	TV_LONG,		// 3
	TV_LONG_LONG,		// 4
	TV_FLOAT,		// 5
	TV_DOUBLE,		// 6
	TV_LONG_DOUBLE,		// 7
	TV_UNKNOWN,		// 8
	TV_LAST};		// 9
	
	
static char *XM_KEYWORD[]={
	"CARTESIAN2D","PARAMETRIC2D","POLAR","CARTESIAN3D",
	"PARAMETRIC3D","CYLINDRIC","SPHERIC",0};

/***************	Instruction	************/
typedef struct Instruction
{
        TObjectX Type;
        char	 *CBrut;
        void	 *aTree;	// Arbre pour la definition brute
        void	 *aEntete;	// Arbre ou ObjectX pour l'entete
}Instruction;

class XMObjectX;

typedef struct SObjectX
{
        TObjectX mType;
        char *Nom;
        void *ObjX;
        void *&ObjectX(){return ObjX;}
        TObjectX &Type(){return mType;}
}SObjectX;

/*************	STree	***************/
typedef struct STree
{
        TObjectX mType; // Type principale
        TObjectX Type;  // Type dans le cas d'un casting
        char *Nom;
        SObjectX *ObjX;
        int Again(){return (Type != TO_INIT);}
}STree;

enum OnState{ON_COMPILE, ON_RUNNING};

/********************************************************************************************/
/*											    */
/********************************************************************************************/

static char *XM_STANDARD_FUNCTIONS[] =
		{"Function",	"Trigonomtrique",	"sin(x)",
		 "Function",	"Trigonomtrique",	"cos(x)",
		 "Function",	"Trigonomtrique",	"tan(x)",
		 0};
		
static char *XM_EASYMATHS_FUNCTIONS[] =
		{"Numeric",	"maths",	"eval(x, x = 1)",
		 "Formel",	"maths",	"evalf(x, x = a)",
		 "Numeric",	"matrice",	"det(M(x))",
		 0};

#endif	/* TYPE_H */
